;(function (document, $) {
    "use strict";

    function initializeValidators() {

        var JOB_NAME_FIELD_ERROR_EL = $('<coral-icon id="unique-name-element" class="coral-Icon coral-Form-fielderror coral-Icon--sizeS coral-Icon--alert" icon="alert" size="S" role="img" aria-label="alert"></coral-icon>'),
            JOB_NAME_TOOLTIP_ERROR_EL = $('<coral-tooltip id="unique-name-element-tooltip" class="coral3-Tooltip coral3-Tooltip--error coral3-Tooltip--arrowLeft" aria-hidden="true" variant="error" tabindex="-1" role="tooltip" style="z-index: 10020; max-width: 1408.84px; left: 164px; top: -3px; display: none;"><coral-tooltip-content>content</coral-tooltip-content></coral-tooltip>');

        $(JOB_NAME_FIELD_ERROR_EL).on("mouseenter", function () {
            $(JOB_NAME_TOOLTIP_ERROR_EL).addClass("is-open");
            $(JOB_NAME_TOOLTIP_ERROR_EL).attr("aria-hidden", "false");
            $(JOB_NAME_TOOLTIP_ERROR_EL).css("display", "");
            $(JOB_NAME_TOOLTIP_ERROR_EL).css("max-width", "1142");
            $(JOB_NAME_TOOLTIP_ERROR_EL).attr("open");
        });

        $(JOB_NAME_FIELD_ERROR_EL).on("mouseleave", function () {
            $(JOB_NAME_TOOLTIP_ERROR_EL).removeClass("is-open");
            $(JOB_NAME_TOOLTIP_ERROR_EL).attr("aria-hidden", "true");
            $(JOB_NAME_TOOLTIP_ERROR_EL).css("display", "none");
            $(JOB_NAME_TOOLTIP_ERROR_EL).css("max-width", "1408");
            $(JOB_NAME_TOOLTIP_ERROR_EL).removeAttr("open");
        });

        var FIELD_ERROR_EL = $('<coral-icon name="config-warn-element" class="coral-Icon coral-Form-fielderror coral-Icon--sizeS coral-Icon--alert" style="float: left;margin-left: 15px;margin-top: 25px;" icon="alert" size="S" role="img" aria-label="alert"></coral-icon>'),
            TOOLTIP_ERROR_EL = $('<coral-tooltip name="config-warn-element-tooltip" class="coral3-Tooltip coral3-Tooltip--error" aria-hidden="true" variant="error" tabindex="-1" role="tooltip" style="z-index: 10020; left: 15px; width: 400px; bottom: 70px; display: none;"><coral-tooltip-content>content</coral-tooltip-content></coral-tooltip>');

        function registerListeners() {
            $(FIELD_ERROR_EL).on("mouseenter", function () {
                TOOLTIP_ERROR_EL.addClass("is-open");
                TOOLTIP_ERROR_EL.attr("aria-hidden", "false");
                TOOLTIP_ERROR_EL.css("display", "");
                TOOLTIP_ERROR_EL.attr("open");
            });

            $(FIELD_ERROR_EL).on("mouseleave", function () {
                TOOLTIP_ERROR_EL.removeClass("is-open");
                TOOLTIP_ERROR_EL.attr("aria-hidden", "true");
                TOOLTIP_ERROR_EL.css("display", "none");
                TOOLTIP_ERROR_EL.removeAttr("open");
            });
        }

        function processErrorResponse(response, reject, form) {
            var fieldWrapper = $('coral-dialog-footer');
            var errorMessage = $.parseJSON(response.responseText).error.details[0];
            TOOLTIP_ERROR_EL.find("coral-tooltip-content").text(errorMessage);
            $('[name="config-warn-element"]').remove();
            $('[name="config-warn-element-tooltip"]').remove();

            fieldWrapper.prepend(TOOLTIP_ERROR_EL);
            fieldWrapper.prepend(FIELD_ERROR_EL);
            registerListeners();

            fieldWrapper.find(".coral-Form-fielderror").show();

            window.smartling.touchui.jobDialog.processValidationError && window.smartling.touchui.jobDialog.processValidationError();

            reject;
        }

        var cloudConfigValidator = function(form) {
            new Promise(function(resolve, reject) {
                $.ajax({
                    url: '/services/smartling/cloud/config/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject, form);
                });
            });
        }

        var autoExecuteValidator = function (form) {
            return new Promise(function(resolve, reject) {
                $.ajax({
                    url: '/services/smartling/auto/execute/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject, form);
                });
            });
        }

        var targetLocalesValidator = function (form) {
            return new Promise(function(resolve, reject) {
                var targetLocalesElement;
                if (form.find('[name="jobUid"]').val()){
                    targetLocalesElement = $('#add-to-job-form [name="targetLocaleCodes"]');
                }else{
                    targetLocalesElement = $('#create-job-form [name="targetLocaleCodes"]');
                }
                $.ajax({
                    url: '/services/smartling/target/locales/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath,
                        targetLocaleCodes: $.map(targetLocalesElement, function(element){return element.value})
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject, form);
                });
            });
        }

        var targetParentPathValidator = function (form) {
            return new Promise(function(resolve, reject) {
                var targetLocalesElement;
                if (form.find('[name="jobUid"]').val()){
                    targetLocalesElement = $('#add-to-job-form [name="targetLocaleCodes"]');
                }else{
                    targetLocalesElement = $('#create-job-form [name="targetLocaleCodes"]');
                }
                $.ajax({
                    url: '/services/smartling/page/parent/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath,
                        targetLocaleCodes: $.map(targetLocalesElement, function(element){return element.value})
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject, form);
                });
            });
        }

        var cloudConfigChildpageValidator = function (form) {
            return new Promise(function(resolve, reject) {
                var isDeep = $(".coral-Panel.is-selected").find("coral-checkbox [name='isDeep']").is(':checked');

                $.ajax({
                    url: '/services/smartling/cloud/config/childpage/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath,
                        isDeep: isDeep
                    },
                    success: resolve
                }).fail(function (response) {
                    processErrorResponse(response, reject, form);
                });
            });
        }

        var jobsNameValidator = function (form) {
            return new Promise(function (resolve, reject) {
                $.ajax({
                    url: '/services/smartling/jobs/name/validator',
                    data: {
                        payload: window.smartling.touchui.jobDialog.pagePath,
                        jobName: form.find('[name="jobName"]').val(),
                        jobUid: form.find('[name="jobUid"]').val()
                    },
                    success: resolve
                }).fail(function (response) {
                    var fieldWrapper = form.find('[name="jobName"]').parent();
                    var errorMessage = $.parseJSON(response.responseText).error.details[0];
                    JOB_NAME_TOOLTIP_ERROR_EL.find("coral-tooltip-content").text(errorMessage);

                    fieldWrapper.append(JOB_NAME_FIELD_ERROR_EL);
                    fieldWrapper.append(JOB_NAME_TOOLTIP_ERROR_EL);

                    form.find('[name="jobName"]').attr("invalid", true);
                    $('#smartling-job-dialog').find('coral-tab[aria-selected="true"]').attr("invalid", true);

                    $("#create-job-form").find('[name="jobName"]').parent().find(".coral-Form-fielderror").show();
                    reject;
                });
            });
        }

        $("#create-job-form").submit(function (e) {
            var form = $("#create-job-form");
            if( form.data("valid") === true) {
                return;
            }

            e.preventDefault();

            Promise.all([cloudConfigValidator(form),
                         autoExecuteValidator(form),
                         targetLocalesValidator(form),
                         cloudConfigChildpageValidator(form),
                         jobsNameValidator(form),
                         targetParentPathValidator(form)]).then(function(values) {
                form.data("valid", true);
                form.submit();
                form.data("valid", false);
            });

            return false;
        });

        $("#add-to-job-form").submit(function (e) {
            var form = $("#add-to-job-form");
            if( form.data("valid") === true) {
                return;
            }

            e.preventDefault();

            Promise.all([cloudConfigValidator(form),
                         autoExecuteValidator(form),
                         targetLocalesValidator(form),
                         cloudConfigChildpageValidator(form),
                         targetParentPathValidator(form)]).then(function(values) {
                form.data("valid", true);
                form.submit();
                form.data("valid", false);
            });

            return false;
        });
    }

    $(document).ready(initializeValidators);

})(jQuery(document), Granite.$);